"use strict"
{
	const C3 = globalThis.C3;

	function CompareValues(va, vb) {
		if (typeof va === "number" && typeof vb === "number") return va - vb
		else {
			const sa = va.toString()
			const sb = vb.toString()
			if (sa < sb) return -1
			else if (sa > sb) return 1
			else return 0
		}
	}

	C3.Behaviors.overboy_array_b.Acts = {
		Clear(v) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) for (let z = 0; z < cz; ++z) arr[x][y][z] = v
		},
		SetSize(w, h, d) {
			this.SetSize(w, h, d)
		},
		SetX(x, val) {
			this.Set(x, 0, 0, val)
		},
		SetXY(x, y, val) {
			this.Set(x, y, 0, val)
		},
		SetXYZ(x, y, z, val) {
			this.Set(x, y, z, val)
		},
		Push(where, value, axis) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (axis === 0) {
				const add = C3.MakeFilledArray(cy, () => C3.MakeFilledArray(cz, value))
				if (where === 0) arr.push(add)
				else arr.unshift(add)
				this._cx++
			} else if (axis === 1) {
				for (let x = 0; x < cx; ++x) {
					const add = C3.MakeFilledArray(cz, value)
					if (where === 0) arr[x].push(add)
					else arr[x].unshift(add)
				}
				this._cy++
			} else {
				for (let x = 0; x < cx; ++x)
					for (let y = 0; y < cy; ++y)
						if (where === 0) arr[x][y].push(value)
						else arr[x][y].unshift(value)
				this._cz++
			}
		},
		Pop(where, axis) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (axis === 0) {
				if (cx === 0) return
				if (where === 0) arr.pop()
				else arr.shift()
				this._cx--
			} else if (axis === 1) {
				if (cy === 0) return
				for (let x = 0; x < cx; ++x)
					if (where === 0) arr[x].pop()
					else arr[x].shift()
				this._cy--
			} else {
				if (cz === 0) return
				for (let x = 0; x < cx; ++x)
					for (let y = 0; y < cy; ++y)
						if (where === 0) arr[x][y].pop()
						else arr[x][y].shift()
				this._cz--
			}
		},
		Reverse(axis) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (cx === 0 || cy === 0 || cz === 0) return
			if (axis === 0) arr.reverse()
			else if (axis === 1) for (let x = 0; x < cx; ++x) arr[x].reverse()
			else for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) arr[x][y].reverse()
		},
		Sort(axis) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (cx === 0 || cy === 0 || cz === 0) return
			if (axis === 0) arr.sort((a, b) => CompareValues(a[0][0], b[0][0]))
			else if (axis === 1) for (let x = 0; x < cx; ++x) arr[x].sort((a, b) => CompareValues(a[0], b[0]))
			else for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) arr[x][y].sort(CompareValues)
		},
		Shuffle(axis) {
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			if (cx === 0 || cy === 0 || cz === 0) return
			if (axis === 0) for (let y = 0; y < cy; ++y) for (let z = 0; z < cz; ++z) this._ShuffleHelper(axis, cx, 0, y, z)
			else if (axis === 1) for (let x = 0; x < cx; ++x) for (let z = 0; z < cz; ++z) this._ShuffleHelper(axis, cy, x, 0, z)
			else for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) this._ShuffleHelper(axis, cz, x, y, 0)
		},
		Delete(index, axis) {
			index = Math.floor(index)
			if (index < 0) return
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (axis === 0) {
				if (index >= cx) return
				arr.splice(index, 1)
				this._cx--
			} else if (axis === 1) {
				if (index >= cy) return
				for (let x = 0; x < cx; ++x) arr[x].splice(index, 1)
				this._cy--
			} else {
				if (index >= cz) return
				for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) arr[x][y].splice(index, 1)
				this._cz--
			}
		},
		Insert(value, index, axis) {
			index = Math.floor(index)
			if (index < 0) return
			const cx = this._cx
			const cy = this._cy
			const cz = this._cz
			const arr = this._arr
			if (axis === 0) {
				if (index > cx) return
				arr.splice(
					index,
					0,
					C3.MakeFilledArray(cy, () => C3.MakeFilledArray(cz, value))
				)
				this._cx++
			} else if (axis === 1) {
				if (index > cy) return
				for (let x = 0; x < cx; ++x) arr[x].splice(index, 0, C3.MakeFilledArray(cz, value))
				this._cy++
			} else {
				if (index > cz) return
				for (let x = 0; x < cx; ++x) for (let y = 0; y < cy; ++y) arr[x][y].splice(index, 0, value)
				this._cz++
			}
		},
		SplitString(str, sep, type) {
			const parts = str.split(sep)
			this.SetSize(parts.length, 1, 1)
			for (let i = 0, len = parts.length; i < len; ++i) {
				let v = parts[i]
				if (type === 0) {
					if (String(Number(v)) === v) v = Number(v)
				} else if (type === 2) v = Number(v)
				this.Set(i, 0, 0, v)
			}
		},
		JSONLoad(json) {
			let o = null
			try {
				o = JSON.parse(json)
			} catch (err) {
				console.error("[Construct] Failed to parse JSON: ", err)
				return
			}
			if (!o["c2array"]) return
			const sz = o["size"]
			this._cx = sz[0]
			this._cy = sz[1]
			this._cz = sz[2]
			this._arr = o["data"]
		},
		JSONDownload(filename) {
			const url = URL.createObjectURL(new Blob([this.GetAsJsonString()], { type: "application/json" }))
			this._runtime.InvokeDownload(url, filename)
		}
	}
}
