const C3 = globalThis.C3;

C3.Behaviors.overboy_array_b.Instance = class overboy_array_bInstance extends globalThis.ISDKBehaviorInstanceBase {
	constructor() {
		super();

		this._cx = 10
		this._cy = 1
		this._cz = 1
		this._arr = null
		this._forX = []
		this._forY = []
		this._forZ = []
		this._forDepth = -1
		const properties = this._getInitProperties();
		if (properties) {
			this._cx = properties[0]
			this._cy = properties[1]
			this._cz = properties[2]
		}
		this._arr = C3.MakeFilledArray(this._cx, () => C3.MakeFilledArray(this._cy, () => C3.MakeFilledArray(this._cz, 0)))
	}

	_release() {
		this._arr = null;
		super._release();
	}

	requireFiniteNumber(e) {
		if ("number" != typeof e || !isFinite(e)) throw new TypeError("expected finite number")
	}

	At(x, y, z) {
		x = Math.floor(x)
		y = Math.floor(y)
		z = Math.floor(z)
		if (x >= 0 && x < this._cx && y >= 0 && y < this._cy && z >= 0 && z < this._cz) return this._arr[x][y][z]
		else return 0
	}
	Set(x, y, z, val) {
		x = Math.floor(x)
		y = Math.floor(y)
		z = Math.floor(z)
		if (x >= 0 && x < this._cx && y >= 0 && y < this._cy && z >= 0 && z < this._cz) this._arr[x][y][z] = val
	}
	SetSize(w, h, d) {
		w = Math.floor(w)
		h = Math.floor(h)
		d = Math.floor(d)
		if (w < 0) w = 0
		if (h < 0) h = 0
		if (d < 0) d = 0
		if (this._cx === w && this._cy === h && this._cz === d) return
		this._cx = w
		this._cy = h
		this._cz = d
		const arr = this._arr
		ResizeArray(arr, w, () => C3.MakeFilledArray(h, () => C3.MakeFilledArray(d, 0)))
		for (let x = 0; x < w; ++x) {
			ResizeArray(arr[x], h, () => C3.MakeFilledArray(d, 0))
			for (let y = 0; y < h; ++y) ResizeArray(arr[x][y], d, 0)
		}
	}
	GetWidth() {
		return this._cx
	}
	GetHeight() {
		return this._cy
	}
	GetDepth() {
		return this._cz
	}
	_ShuffleHelper(axis, len, x, y, z) {
		while (len > 0) {
			const i = Math.floor(this._runtime.Random() * len)
			--len
			if (axis === 0) {
				const v1 = this.At(len, y, z)
				const v2 = this.At(i, y, z)
				this.Set(len, y, z, v2)
				this.Set(i, y, z, v1)
			} else if (axis === 1) {
				const v1 = this.At(x, len, z)
				const v2 = this.At(x, i, z)
				this.Set(x, len, z, v2)
				this.Set(x, i, z, v1)
			} else if (axis === 2) {
				const v1 = this.At(x, y, len)
				const v2 = this.At(x, y, i)
				this.Set(x, y, len, v2)
				this.Set(x, y, i, v1)
			}
		}
	}

	_getDebuggerProperties() {
		const prefix = "plugins.arr.debugger"
		const propsPrefix = "plugins.arr.properties"
		const ret = [
			{
				title: prefix + ".array-properties.title",
				properties: [
					{
						name: propsPrefix + ".width.name",
						value: this._cx,
						onedit: (v) => this.SetSize(v, this._cy, this._cz)
					},
					{
						name: propsPrefix + ".height.name",
						value: this._cy,
						onedit: (v) => this.SetSize(this._cx, v, this._cz)
					},
					{
						name: propsPrefix + ".depth.name",
						value: this._cz,
						onedit: (v) => this.SetSize(this._cx, this._cy, v)
					},
					{
						name: propsPrefix + ".elements.name",
						value: this._cx * this._cy * this._cz
					}
				]
			}
		]
		const dataProps = []
		if (this._cy === 1 && this._cz === 1)
			for (let x = 0; x < this._cx; ++x)
				dataProps.push({
					name: "$" + x,
					value: this._arr[x][0][0],
					onedit: (v) => (this._arr[x][0][0] = v)
				})
		else
			for (let x = 0; x < this._cx; ++x)
				dataProps.push({
					name: "$" + x,
					value: this._arr[x].toString()
				})
		if (dataProps.length)
			ret.push({
				title: prefix + ".array-data.title",
				properties: dataProps
			})
		return ret
	}

	GetAsJsonString() {
		return JSON.stringify({
			c2array: true,
			size: [this._cx, this._cy, this._cz],
			data: this._arr
		})
	}
	_saveToJson() {
		return { size: [this._cx, this._cy, this._cz], data: this._arr }
	}
	_loadFromJson(o) {
		const sz = o["size"]
		this._cx = sz[0]
		this._cy = sz[1]
		this._cz = sz[2]
		this._arr = o["data"]
	}
	_GetForX() {
		if (this._forDepth >= 0 && this._forDepth < this._forX.length) return this._forX[this._forDepth]
		else return 0
	}
	_GetForY() {
		if (this._forDepth >= 0 && this._forDepth < this._forY.length) return this._forY[this._forDepth]
		else return 0
	}
	_GetForZ() {
		if (this._forDepth >= 0 && this._forDepth < this._forZ.length) return this._forZ[this._forDepth]
		else return 0
	}

	get width() {
		return this.GetWidth()
	}
	get height() {
		return this.GetHeight()
	}
	get depth() {
		return this.GetDepth()
	}

	setSize(w, h = 1, d = 1) {
		this.requireFiniteNumber(w)
		this.requireFiniteNumber(h)
		this.requireFiniteNumber(d)
		this.SetSize(w, h, d)
	}
	getAt(x, y = 0, z = 0) {
		this.requireFiniteNumber(x)
		this.requireFiniteNumber(y)
		this.requireFiniteNumber(z)
		return this.At(x, y, z)
	}
	setAt(val, x, y = 0, z = 0) {
		this.requireFiniteNumber(x)
		this.requireFiniteNumber(y)
		this.requireFiniteNumber(z)
		if (typeof val !== "number" && typeof val !== "string") throw new TypeError("invalid type")
		this.Set(x, y, z, val)
	}
}

function ResizeArray(arr, len, data) {
	if (len < arr.length) C3.truncateArray(arr, len)
	else if (len > arr.length)
		if (typeof data === "function") for (let i = arr.length; i < len; ++i) arr.push(data())
		else for (let i = arr.length; i < len; ++i) arr.push(data)
}
